(function (Drupal, drupalSettings, klaro) {

  'use strict';

  /**
   * Initialize Klaro! with custom settings.
   */
  Drupal.behaviors.klaro = {
    attach: function (context, settings) {
      if (!settings.klaro || !klaro) {
        return;
      }

      // Setup klaro on each attach (support ajax insert commands).
      klaro.setup(settings.klaro.config);

      // (Re-)apply consents on each attach (ajax add_js support)
      // only if there are elements with data-src left.
      if (document.querySelectorAll('[data-src]').length > 0) {
        klaro.getManager().applyConsents();
      }

      if(context === document) {

        var elements = once('klaro', 'body', context);
        Array.prototype.forEach.call(elements, function () {
          // Create regular expressions from config.
          settings.klaro.config.services.forEach(function (app, a_index) {
            app.cookies.forEach(function (cookie, c_index) {
              settings.klaro.config.services[a_index]['cookies'][c_index] = [new RegExp(cookie[0], 'i'), cookie[1] || '/', cookie[2] || location.host];
              settings.klaro.config.services[a_index].callback = Drupal.behaviors.klaro.attachBehaviors;
            });
          });

          // Add toggle dialog button.
          if (settings.klaro.show_toggle_button && !document.getElementById('klaro_toggle_dialog')) {
            var button_html = '<div id="klaro_toggle_dialog" class="klaro_toggle_dialog klaro_toggle_dialog_override" rel="open-consent-manager" ></div>';
            document.body.insertAdjacentHTML('afterend', button_html);
          }

        });
      }

    },

    /**
     * Attach Drupal behaviors from the just consented file(s).
     */
    attachBehaviors: function (consent, app) {

      if (!consent) {
        return;
      }
      if (app.onlyOnce && klaro.getManager(drupalSettings.klaro.config).executedOnce[app.name]) {
        return;
      }

      let knownBehaviors = Object.keys(Drupal.behaviors);
      var elements = document.querySelectorAll('[data-name="' + app.name + '"][src]');
      if (!elements) {
        return;
      }
      function processBehaviours() {
        for (let name in Drupal.behaviors) {
          if (knownBehaviors.indexOf(name) === -1) {
            // Execute newly added behaviors. This is somewhat guesswork, as
            // the order of execution is undetermined based on file load order.
            Drupal.behaviors[name].attach(document, drupalSettings);
            knownBehaviors.push(name);
          }
        }
      }
      Array.prototype.forEach.call(elements, function(element) {
        if (element.complete) {
          processBehaviours();
        }
        else {
          element.addEventListener('load', processBehaviours);
        }
      });
    },
  };

  /**
   * Open the Klaro! consent notice from links & buttons.
   */
  Drupal.behaviors.klaroLink = {
    attach: function (context, settings) {
      var elements = once('klaroLink', '[rel*="open-consent-manager"]', context);
      Array.prototype.forEach.call(elements, function(element) {
        element.addEventListener('click', function (event) {
          klaro.show(settings.klaro.config);
          event.preventDefault();
          return false;
        });
      });
    }
  };
})(Drupal, drupalSettings, klaro);
