<?php

namespace Drupal\klaro\EventSubscriber;

use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpKernel\Event\ResponseEvent;
use Symfony\Component\HttpKernel\KernelEvents;
use Symfony\Component\HttpKernel\HttpKernelInterface;
use Drupal\klaro\Utility\KlaroHelper;

/**
 * Subscribes to the kernel response event.
 */
class ResponseSubscriber implements EventSubscriberInterface {

  /**
   * The Klaro Helper instance.
   *
   * @var \Drupal\klaro\Utility\KlaroHelper
   */
  protected $klaroHelper;

  /**
   * The constructor.
   *
   * @param \Drupal\klaro\Utility\KlaroHelper $klaro_helper
   *   The messenger service.
   */
  public function __construct(KlaroHelper $klaro_helper) {
    $this->klaroHelper = $klaro_helper;
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    $events[KernelEvents::RESPONSE][] = 'onKernelResponse';
    return $events;
  }

  /**
   * Let klaroHelper modify the Html of the response.
   *
   * @param \Symfony\Component\HttpKernel\Event\ResponseEvent $event
   *   The Response event.
   */
  public function onKernelResponse(ResponseEvent $event) {
    // Do nothing if not activated.
    if (!$this->klaroHelper->getSettings()->get('auto_decorate_final_html')) {
      return;
    }

    // Do nothing if on disabled uri.
    // @todo how to determine this for ajax-requests?
    if ($this->klaroHelper->onDisabledUri($event->getRequest())) {
      return;
    }

    // Do nothing if not main request.
    if ($event->getRequestType() !== HttpKernelInterface::MASTER_REQUEST) {
      return;
    }

    $response = $event->getResponse();

    // Modify AjaxCommand responses.
    if (get_class($response) === 'Drupal\Core\Ajax\AjaxResponse') {
      foreach ($response->getCommands() as &$cmd) {
        $cmd = $this->klaroHelper->handleAjaxCommand($cmd);
      }
      return;
    }

    // @todo maybe only process if content-type header has text/html?
    $content = $response->getContent();
    if (!empty($content)) {
      $response->setContent($this->klaroHelper->processHtml($content));
    }
  }

}
